﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.GroupingVector {
  [Item("Grouping Vector One Position Manipulator", "An operator which manipulates a PackingSequence representation.")]
  [StorableClass]
  public class GroupingVectorOnePositionManipulator :  GroupingVectorManipulator {
    [StorableConstructor]
    protected GroupingVectorOnePositionManipulator(bool deserializing) : base(deserializing) { }
    protected GroupingVectorOnePositionManipulator(GroupingVectorOnePositionManipulator original, Cloner cloner) : base(original, cloner) { }
    public GroupingVectorOnePositionManipulator()
      : base() {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new GroupingVectorOnePositionManipulator(this, cloner);
    }

    protected override void Manipulate(IRandom random, GroupingVectorEncoding individual) {
      int max = 0;
      foreach (int i in individual.GroupingVector)
        if (i > max) max = i;

      int index = random.Next(individual.GroupingVector.Length);
      individual.GroupingVector[index] = random.Next(max);
    }

  }
}
